/**
 * \mainpage Digitális áramkörök modellezése
 *
 * \par Feladat:
 * Digitális áramkör modellezése, szimulálása. Megvalósított objektumok:
 *  - Drót (Drot)
 *  - Csomópont (Csp)
 *  - Kapcsoló (Kapcsolo)
 *  - Forrás (Forras)
 *  - Nyomógomb (Gomb)
 *  - NAND kapu
 *  - Inverz bemenetű RS flip-flop (R_S_FF)
 *
 * \par Demostrációs cél:
 *  - öröklés és többszörös öröklés megvalósításának bemutatása
 *  - virtuális tagfüggvény bemutatása
 * \par Hivatkozások
 *  - A program forrása letölthető <A href="https://git.ik.bme.hu/Prog2/eloadas_peldak/ea_06">innen.</a>
 *  - A megértést segíti a 6. előadás anyaga.
 *
 * \par Hiányosságok:
 *  - A jelkezelés nem használja a nem definiált állapotot
 *  - A kapcsolódási pontoknak (lábszámoknak) semmi köze a tényleges áramköri elem lábsorszámához
 *  - A lábakon megjelenő tároló lehetne a Conn objektumban is.
 *
 */


#include <iostream>
#include "forras.h"
#include "kapcsolo.h"
#include "r_s_ff.h"
#include "nand.h"

using std::cerr;
using std::endl;
using std::cin;

/**
 * Egyszerű tesztkapcsolás
 * 2 forrás, 2 kapcsoló, RS flop-flop.
 *
 */
int main()
{
    Kapcsolo K1("K1"), K2("K2");    // kapcsolók
    Forras F1("F1"), F2("F2");      // források
    R_S_FF FF("FF");                // RS FF

    try {
      F1.setConn(0, K1, 0);  // F1 láb 0 -> K1 láb 0
      FF.setConn(0, K1, 1);  // FF láb 0 -> K1 láb 1
      F2.setConn(0, K2, 0);  // F2 láb 0 -> K2 láb 0
      FF.setConn(1, K2, 1);  // FF láb 1 -> K1 láb 1

      cerr << "\nInit:"; F1.init(); F2.init(); // Forrásokat "bekapcsoljuk"
      cerr << "\n\nS_  R_  Q   Q_";
#ifdef DEBUG
      cerr << endl;
#endif // DEBUG
      K1.be(); K2.be();
      cerr << "\n" << FF.get(0).J << "   "  << FF.get(1).J << "   " << FF.get(2).J  << "   " << FF.get(3).J;
cin.ignore(1);
      K1.ki(); K2.be();
      cerr << "\n" << FF.get(0).J << "   "  << FF.get(1).J << "   " << FF.get(2).J  << "   " << FF.get(3).J;
cin.ignore(1);
      K1.be(); K2.be();
      cerr << "\n" << FF.get(0).J << "   "  << FF.get(1).J << "   " << FF.get(2).J  << "   " << FF.get(3).J;
cin.ignore(1);
      K1.be(); K2.ki();
      cerr << "\n" << FF.get(0).J << "   "  << FF.get(1).J << "   " << FF.get(2).J  << "   " << FF.get(3).J;
cin.ignore(1);
      K1.ki(); K2.ki();
      cerr << "\n" << FF.get(0).J << "   "  << FF.get(1).J << "   " << FF.get(2).J  << "   " << FF.get(3).J;

      cerr << "\nHurok kovetkezik:" << endl;
cin.ignore(1);
      NAND Nx("Nx");
      F1.setConn(0, Nx, 0);
      Nx.setConn(2, Nx, 1);   // saját magához kötöttük
      F1.init();              // és belerúgunk
    } catch (const char *s) {
        cerr << "Elkaptuk: " << s << '\n';

    }
    return(0);
}
